@@ -116,6 +116,19 @@ namespace Model
     GenerateDataKeyPairResult& WithCiphertextForRecipient(CiphertextForRecipientT&& value) { SetCiphertextForRecipient(std::forward<CiphertextForRecipientT>(value)); return *this;}
     ///@}
 
+    ///@{
+    /**
+     * <p>The identifier of the key material used to encrypt the private key. This
+     * field is omitted if the request includes the <code>Recipient</code>
+     * parameter.</p>
+     */
+    inline const Aws::String& GetKeyMaterialId() const { return m_keyMaterialId; }
+    template<typename KeyMaterialIdT = Aws::String>
+    void SetKeyMaterialId(KeyMaterialIdT&& value) { m_keyMaterialIdHasBeenSet = true; m_keyMaterialId = std::forward<KeyMaterialIdT>(value); }
+    template<typename KeyMaterialIdT = Aws::String>
+    GenerateDataKeyPairResult& WithKeyMaterialId(KeyMaterialIdT&& value) { SetKeyMaterialId(std::forward<KeyMaterialIdT>(value)); return *this;}
+    ///@}
+
     ///@{
     
     inline const Aws::String& GetRequestId() const { return m_requestId; }
@@ -144,6 +157,9 @@ namespace Model
     Aws::Utils::ByteBuffer m_ciphertextForRecipient{};
     bool m_ciphertextForRecipientHasBeenSet = false;
 
+    Aws::String m_keyMaterialId;
+    bool m_keyMaterialIdHasBeenSet = false;
+
     Aws::String m_requestId;
     bool m_requestIdHasBeenSet = false;
   };
